#!/usr/bin/python

## expendable

## Copyright (C) 2008, 2009, 2011 Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
if __name__ == '__main__':
    sys.path.append (os.environ.get ('top_srcdir', '.'))

from gettext import gettext as _
import gtk
import account

name=_("Payment in installments")
description=_("A one-off withdrawal, or a series of withdrawals.")

class Model(account.Model):
    def __init__ (self, amount, term=1):
        account.Model.__init__ (self)
        self.amount = amount
        self.term = term
        self.payable = amount * term

    def cash_available (self):
        if self.per < self.term:
            return -self.payable

        return 0.0

    def cash_available_next_month (self):
        if self.per == 0:
            liabilities = -self.amount
        else:
            liabilities = 0.0
        return liabilities

    def iter (self):
        if self.parent == None:
            return False

        amount = self.parent.adjust (-self.amount, self)
        if amount < self.amount:
            raise account.ModelError, (self, "Not enough for payment")

        self.payable -= amount
        self.model_data.append( { 'month': self.per,
                                  'amount': self.amount,
                                  'payable': self.payable} )
        if self.payable < 0.01:
            return True

        return False

    def get_datasets (self):
        model_data = self.model_data
        pdata = map (lambda x: (x['month'], x['amount']), model_data)
        return [{'name': self.account.get_name (),
                 'data': pdata}]

class Data(account.Data):
    required_tags = ['amount']
    uixml = "payment.ui"

    def __init__ (self, domain=None):
        account.Data.__init__ (self)
        for dir in sys.path:
            f = os.path.join (dir, self.uixml)
            if os.access (f, os.R_OK):
                self.builder = gtk.Builder ()
                if domain:
                    self.builder.set_translation_domain (domain)

                self.builder.add_from_file (f)
                break

        data = {}
        for name in ['amount']:
            widget = self.builder.get_object (name)
            data[name] = widget.get_value ()
        for name in ['term']:
            widget = self.builder.get_object (name)
            data[name] = int (widget.get_value ())
        self.data = data

    def is_leaf_node (self):
        return True

    def read (self, xml):
        data = {'term': 1}
        
        for child in xml.getchildren ():
            # Floats
            if child.tag in ['amount']:
                data[child.tag] = float (child.text)
            elif child.tag in ['term']:
                t = child.text
                dot = t.find (".")
                if dot != -1:
                    t = t[:dot]
                data[child.tag] = int (t)
            else:
                print "Not understood: %s" % child.tag

        for required in self.required_tags:
            if not data.has_key (required):
                raise RuntimeError, "Required tags: %s" % self.required_tags

        self.data = data
        self._set_last_known_good ()

    def write (self):
        xml = "<payment>\n"
        for var in self.required_tags:
            value = self.data[var]
            if type (value) == float:
                value = "%.4f" % value
            xml += "  <%s>%s</%s>\n" % (var, value, var)
        for var in ['term']:
            if self.data.has_key (var):
                value = self.data[var]
                if var == 'term' and value == 1:
                    # Don't write this as it is the default value.  This
                    # preserves a bit more data file compatibility between
                    # versions.
                    continue
                if type (value) == float:
                    value = "%.4f" % value
                xml += "  <%s>%s</%s>\n" % (var, value, var)
        xml += "</payment>\n"
        return xml

    def ui (self, variables_vbox, results_vbox):
        w = self.builder.get_object ('variables_table')
        w.reparent (variables_vbox)
        for var in self.required_tags:
            widget = self.builder.get_object (var)
            widget.set_value (self.data[var])
            widget.connect ("value-changed", self.variable_changed, var)
        for var in ['term']:
            widget = self.builder.get_object (var)
            if self.data.has_key (var):
                widget.set_value (self.data[var])
            widget.connect ("value-changed", self.variable_changed, var)

    def variable_changed (self, widget, var):
        value = widget.get_value ()
        if var == 'term':
            value = int (value)
        self.data[var] = value
        try:
            self.redisplay ()
        except:
            widget.set_value (self.last_known_good[var])

    def get_model (self):
        return Model (**self.data)

if __name__ == '__main__':
    from pprint import *
    from xml.etree.ElementTree import XML
    d = Data ()
    w = d.write ()
    xml = XML (w)
    d.read (xml)

    import savings
    s = savings.Model (rate=0,
                       balance=1000,
                       taxrate=0)
    payment = Model (amount=1000)
    payment.set_delay (3)
    s.add_model (payment)
    s.run (nper=8)
    data = s.get_model_data ()
    if abs (data[4]['balance']) > 0.01:
        raise RuntimeError, "Too much paid"
